#!/usr/bin/python
"""Print out available grass locations, mapsets, cells and image groups.

Usage: grassinfo [-D database] [-L location] [-M mapset]
                  [-l] [-m] [cellname]

Converted from libgrass5 v0.9.0 example program grassinfo.c by Frank Warmerdam.
Improved.
Command line parameters are different, though.

Copyright 2000 ImageLinks
Author: Bernhard Reiter <bernhard@intevation.de>
This script is part of libgrass5 and free software under the GNU LGPL.
"""

__version__="$Revision: 1.2 $"

from pylibgrass5 import *
import getopt
import sys


def currentposition():
    """ Returns tuple of current gisdbase, location and mapset. 
    
    Should always return values after a successful G_gisinit2 call.
    """
    return (G__getenv("GISDBASE"),
            G__getenv("LOCATION_NAME"),
            G__getenv("MAPSET"))

def get_cellinfo(given_cellname,cell_header):
    """ Fill cell_header object and return cell description. 
    
    cell_header has to already exist 
    """
    status,mapset,cellname=G_check_cell(given_cellname)
    if not status:
        sys.stderr.write(
            "Not able to open cellfile for [%s]\n"%(given_cellname))
        #sys.exit(1)

    s=""
    G_get_cellhd(cellname,mapset,cell_header)
    data_type,cellname=G_raster_map_type(cellname,mapset)

    if data_type== CELL_TYPE:
        s=s+'\n'+ "data type = integer (CELL)"
    elif data_type == FCELL_TYPE:
        s=s+'\n'+ "data type = float (FCELL)"
    elif data_type == DCELL_TYPE:
        s=s+'\n'+ "data type = double (DCELL)"

    s=s+'\n'+ "bytes per pixel = %d" %(cell_header.format+1 )
    s=s+'\n'+ "compressed = %d" %(cell_header.compressed )
    s=s+'\n'+ "rows, cols = %dx%d" %(cell_header.rows, cell_header.cols )
    s=s+'\n'+ "proj = %d" %(cell_header.proj )
    s=s+'\n'+ "zone = %d" %(cell_header.zone )
    s=s+'\n'+ "ew_res = %f" %(cell_header.ew_res )
    s=s+'\n'+ "ns_res = %f" %(cell_header.ns_res )
    s=s+'\n'+ "north = %f" %(cell_header.north )
    s=s+'\n'+ "south = %f" %(cell_header.south )
    s=s+'\n'+ "east = %f" %(cell_header.east )
    s=s+'\n'+ "west = %f" %(cell_header.west )

    # Try to return a PROJ.4 formatted coordinate system.

    s=s+'\n'+ "'%s'"%(cellname)
    p4_def = G_get_cell_as_proj4( cellname, mapset )
    if not p4_def:
        s=s+'\n'+ "PROJ.4 Definition = <not available>\n"
    else:
        s=s+'\n'+ "PROJ.4 Definition = %s\n"%(p4_def)

    return s


def get_imagerygrouplist(group_name):
    ref= Ref()
    l=[]
    status=I_get_group_ref(group_name,ref)

    for iref in range(0,ref.nfiles):
        n= ref.file[iref].name;
        m= ref.file[iref].mapset;
        cellname="%s@%s" % (n,m)
        l.append(cellname)

    del ref # not really necessary, but a test
    return l


def main():
    gisdbase=None
    location=None
    mapset=None
    group_name=None
    user_cellname=None

    list_locations = 0
    list_mapsets = 0

    # handle command line arguments

    try:
        optlist, args=getopt.getopt(sys.argv[1:],"D:lL:mM:")
    except getopt.error,x:
        sys.stdout.write(x+"\n\n")
        print __doc__
        sys.exit(1)

    for opt in optlist:
        if opt[0] in ('-D',):
            gisdbase=opt[1]

        elif opt[0] in ('-L',):
            location=opt[1]

        elif opt[0] in ('-M',):
            mapset=opt[1]

        elif opt[0] in ('-l',):
            list_locations=1

        elif opt[0] in ('-m',):
            list_mapsets=1

        else:
            # should not happen!
            print "Program inconsistancy detected! Bailing out..."
            sys.exit(10)

    if len(args)>1:
        sys.stdout.write("Only one cellname allowed!\n\n")
        print__doc__
        sys.exit(1)
    else:
        if len(args)==1:
            user_cellname=args[0]

    # if we just have the database given, we can only list the locations.
    #if gisdbase and not location:
    #    list_locations=1

    
    # Initialize base on the selected (or default) mapset.
    error=G_gisinit_2("grassinfo.py",gisdbase,location,mapset)
    if error:
        print error
    print "Database: %s  Location: %s  Mapset: %s" % currentposition()

    if list_locations:
        list=G_fetch_list_locations(gisdbase)
        print "Possible location names:"
        for location in list:
            print location

    if list_mapsets:
        if location:
            list=G_fetch_list_mapsets(None)
            print "Possible mapset names:"
            for map in list:
                print map
        else:
            sys.stdout.write("Warning: "+\
                "Can only list mapsets, if location is known.\n")
            sys.exit(2)


    # we might be done here
    if list_mapsets or list_locations:
        sys.exit(0)


    if error and not user_cellname:
        sys.stdout.write("Unable to initialize database access, "+\
                         "no valid .grassrc5\n")
        sys.exit(1)

    # Did the user select a cell file?  If not, list available ones. 
    if not user_cellname:
        cellhds=G_fetch_list_element("cellhd","")
        print "Possible cell names:"
        for cellhd in cellhds:
            print cellhd

        groups=G_fetch_list_element("group", "")
        print "Possible imagery group names:"
        for group in groups:
            print group

        sys.exit(0)

    cellnames=[]

    status,mapset,group_name=I_check_group(user_cellname)
#    print status,mapset, group_name
    if status:
        print "Open Imagery Group '%s'." % (group_name)
        print

        celllist=get_imagerygrouplist(group_name)

        for cellname in celllist:
            print cellname

        cellnames=celllist

    else:
        cellnames.append(user_cellname)
     
    cell_header=Cell_head()

    for cellname_from_list in cellnames:
        print
        print get_cellinfo(cellname_from_list,cell_header)
        print 

#        # Actually open the cell. This is just a test to find out if we can.

        G_set_window( cell_header );

        cellfile = G_open_cell_old_2( cellname_from_list );
    
        if cellfile < 0:
            sys.stderr.write(
                "Not able to open cellfile for [%s]\n"%( cellname_from_list))
            sys.exit(1)

        G_close_cell( cellfile );

if __name__ == "__main__":
    main()
